<?php

namespace Drupal\fieldupdate\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Url;

/**
 * Class ForgotUsernameForm.
 */
class FieldUpdateForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'fieldupdate';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
//    $field = FieldStorageConfig::loadByName('node', 'my_field');
//    $upfield = FieldConfig::loadByName('node', 'employee' , 'field_text_formated_long');
//    $upfield = FieldConfig::loadByName('node', 'employee' , 'field_category_test');
//    $upfield = FieldConfig::loadByName('node', 'employee' , 'body');
//    p($upfield);
//    $settings = $upfield->getFieldDefinition()->getSettings();
//p($settings);

    $entity_types_options = [
      'node' => $this->t('Content'),
      'taxonomy_term' => $this->t('Taxonomy term'),
      'user' => $this->t('User')
    ];
    $selected_entity = $form_state->getValue('entity_types') ?: '';
    $selected_entity = $form_state->getValue('entity_types') ?: 'node';


    $form['entity_types'] = [
      '#title' => t('Entity type'),
      '#type' => 'select',
      '#options' => $entity_types_options,
      '#empty_option' => $this->t('- None -'),
      '#ajax' => [
        'callback' => '::bundleDropdownCallback',
        'wrapper' => 'bundle-field-wrapper',
      ],
      '#default_value' => $selected_entity,
    ];

    $form['depended_fieldset_container'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'bundle-field-wrapper'],
    ];

//    $selected_bundle = $form_state->getValue('bundle') ?: '';

    $form['depended_fieldset_container']['bundle'] = [
      '#type' => 'select',
      '#title' => $this->t('Bundle'),
      '#options' => $this->getSecondDropdownOptions($selected_entity),
//      '#default_value' => !empty($form_state->getValue('bundle')) ? $form_state->getValue('bundle') : '',
      '#default_value' => !empty($form_state->getValue('bundle')) ? $form_state->getValue('bundle') : 'employee',
      '#empty_option' => $this->t('- None -'),
      '#ajax' => [
        'callback' => '::fieldsDropdownCallback',
        'wrapper' => 'fields-field-wrapper',
      ],
    ];

    $selected_bundle = $form_state->getValue('bundle') ?: 'employee';

    $form['depended_fieldset_container']['depend_next'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'fields-field-wrapper'],
    ];
    $form['depended_fieldset_container']['depend_next']['field'] = [
      '#type' => 'select',
      '#title' => $this->t('Field List') . $form_state->getValue('field'),
      '#options' => static::getThirdDropdownOptions($selected_entity, $selected_bundle),
      '#default_value' => !empty($form_state->getValue('bundle')) ? $form_state->getValue('bundle') : '',
      '#empty_option' => $this->t('- None -'),
      '#ajax' => [
        'callback' => '::fieldsForUpdate',
        'wrapper' => 'entity-field-wrapper',
      ],
    ];


    $form['entity_field_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'entity-field-wrapper'],
    ];

    $selected_field = $form_state->getValue('field') ?: '';

    if ($selected_entity && $selected_field) {

      $upfield = FieldConfig::loadByName($selected_entity, $selected_bundle , $selected_field);

      $form['entity_field_wrapper']['update_field'] = $this->renderFormFieldForUpdate($upfield);
    }

    $form['submit'] = [
      '#value' => t('Update'),
      '#type' => 'submit',
    ];

    return $form;
  }

  public function bundleDropdownCallback(array $form, FormStateInterface $form_state) {
    return $form['depended_fieldset_container'];
  }
  public function fieldsDropdownCallback(array $form, FormStateInterface $form_state) {
    return $form['depended_fieldset_container']['depend_next'];
  }
  public function fieldsForUpdate(array $form, FormStateInterface $form_state) {
    return $form['entity_field_wrapper'];
  }


  public function getSecondDropdownOptions($key = '') {
    $types = [];
    $entity_machine_name = [
      'node' => 'node_type',
      'taxonomy_term' => 'taxonomy_vocabulary',
    ];
//    \Drupal::logger('ddd')->info('here-' . $key . '>>>>' . $entity_machine_name[$key]);
    if ($key && array_key_exists($key, $entity_machine_name)) {
      \Drupal::logger('ddd')->info('GO');
      $entityTypeManager = \Drupal::service('entity_type.manager');
      $contentTypes = $entityTypeManager->getStorage($entity_machine_name[$key])->loadMultiple();
      foreach ($contentTypes as $contentType) {
        $types[$contentType->id()] = $contentType->label();
      }
    }
    return $types;
  }

  function getThirdDropdownOptions($selected_entity, $bundle = '') {
    $types = [];
    if ('user' == $selected_entity) {
      $bundle = $selected_entity;
    }
    if ($selected_entity && $bundle) {
      $entityFieldManager = \Drupal::service('entity_field.manager');
      $fields = $entityFieldManager->getFieldDefinitions($selected_entity, $bundle);
      foreach ($fields as $key => $field) {
        $info = \Drupal\field\Entity\FieldStorageConfig::loadByName($selected_entity, $key);
        if ($info) {
          // kint($field->getOriginalId());
          $types[$field->getName()] = $field->getLabel() . " (" . $field->getName() . ")";
        }
      }

    }
    return $types;
  }

  private function renderFormFieldForUpdate($field_obj) {
    $field_label = $field_obj->getLabel();
    $field_name = $field_obj->getName();
    $field_type = $field_obj->getType();

    $drupal_fields = [
      'image' => 'managed_file',
      'string' => 'textfield',
      'boolean' => 'checkbox',
      'text_long' => 'text_format',
      'text_with_summary' => 'text_format',
      'entity_reference' => 'select',
    ];
//    return $drupal_fields[$field_type];

    if (array_key_exists($field_type, $drupal_fields)) {
      return [
        '#type' => $drupal_fields[$field_type],
        '#title' => $field_label,
        '#attributes' => ['id' => 'entity-field-wrapper'],

      ];
    }

    return [];

  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $post = $form_state->getValues();
//    p($post);

    $entity_type = $post['entity_types'];
    $bundle = $post['bundle'];
    $field_machine_name = $post['field'];
    $field_data = $post['update_field'];
    $field_type = FieldConfig::loadByName('node', 'employee' , 'field_img')->getType();

    $query = \Drupal::entityQuery($entity_type);
    if ('node' == $entity_type) {
      $query->condition('type', $bundle);
    }
    if ('taxonomy_term' == $entity_type) {
      $query->condition('vid', $bundle);
    }
    $query->notExists($field_machine_name);
    $query->range(0,1);
    $results = $query->execute();
    $total = count($results);

    $batch = [
      'title' => t('Importing animals'),
      'operations' => [],
      'init_message' => t('Import process is starting.'),
      'progress_message' => t('Processed @current out of @total. Estimated time: @estimate.'),
      'error_message' => t('The process has encountered an error.'),
    ];

    foreach($results as $entity_id) {
      $dt = [
        'entity_id' => $entity_id,
        'value' => $field_data,
        'field_machine_name' => $field_machine_name,
        'field_type' => $field_type,
      ];
      $batch['operations'][] = [
        ['\Drupal\fieldupdate\Form\FieldUpdateForm', 'fieldupdate_operations'],
        [$dt]
      ];
    }
    batch_set($batch);
    \Drupal::messenger()->addMessage('Imported ' . $total . ' animals!');

    $form_state->setRebuild(TRUE);

  }

  /**
   * @param $entity
   * Deletes an entity
   */
  public static function fieldupdate_operations($data, &$context) {
    $node = \Drupal\node\Entity\Node::load($data['entity_id']);

    // Testing done for
    /*
     string => textfield, boolean => Single checkbox
     */
    $save_data = $data['value'];
    if ('image' == $data['field_type']) {
      $save_data[] = [
        'target_id' => 1,
        'alt' => 'Dummy alt',
        'title' => 'Dummy title',
      ];
    }
    if ('text_long' == $data['field_type']) {
      $save_data = ['value' => $data['value'], 'format' => $data['format']];
    }
    $node->set($data['field_machine_name'], $save_data);
    $node->save();

    \Drupal::logger('batch test')->info(t('Success-@id', ['@id' => $node->id()]));
    $context['results'][] = $node->getTitle();
    $context['message'] = t('Created @title', array('@title' => $node->getTitle()));
  }


}
